// Importing necessary functions from d3
import {
  csv,
  select,
  pie,
  arc,
  scaleOrdinal,
  format,
} from 'd3';

// Assigning colors for the pie chart sections
const sectionColors = [
  'gold',
  '#2788bc',
  'Pink',
  'crimson',
  'green',
  'slateblue',
  'chocolate',
  'blue',
  'lightgreen',
  'aqua',
];

// Dataset URL
const csvUrl =
  'https://gist.githubusercontent.com/hemalatha1700/422a3d90f4d27b9be8bc552bd3ffb53c/raw/a165b22d911af94728e549f5771eb1fbc275c9f6/movies.csv';

// Function to fetch data from the dataset
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};

// SVG setup: height and width
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

// Formatter for pie chart numbers
const xAxisTickFormat = (number) => format('.4~g')(number);

// Function to render data on the pie chart
const render = (data) => {
  // Preparing data for pie chart construction
  const pieData = pie().value((d) => d.IMDb_Rating)(data);

  // Setting up color scale
  const colors = scaleOrdinal()
    .domain(data.map((d) => d.IMDb_Rating))
    .range(sectionColors);

  // Arc generator for the pie chart sections
  const segments = arc().innerRadius(0).outerRadius(175);

  // Constructing pie chart sections
  const sections = svg
    .append('g')
    .attr(
      'transform',
      `translate(${width / 2},${height / 2})`,
    )
    .selectAll('path')
    .data(pieData);

  sections
    .enter()
    .append('path')
    .attr('d', segments)
    .attr('fill', (d) => colors(d.data.IMDb_Rating));

  // Adding labels to each section
  sections
    .enter()
    .append('text')
    .text((d) => `$${xAxisTickFormat(d.data.IMDb_Rating)}`)
    .attr(
      'transform',
      (d) => `translate(${segments.centroid(d)})`,
    )
    .style('text-anchor', 'middle')
    .style('fill', 'black')
    .style('font-size', '15px');

  // Constructing the legend
  const legendsXTranslation = width - 300; // Adjusted to accommodate longer text
  const legend = svg
    .append('g')
    .attr(
      'transform',
      `translate(${legendsXTranslation},70)`,
    )
    .selectAll('.legend')
    .data(pieData)
    .enter()
    .append('g')
    .attr(
      'transform',
      (d, i) => `translate(0,${(i + 1) * 30})`,
    );

  // Adding colored squares to the legend
  legend
    .append('rect')
    .attr('width', 20)
    .attr('height', 20)
    .attr('fill', (d) => colors(d.data.IMDb_Rating));

  // Adjusting the position of movie titles and net worth in the legend
  legend
    .append('text')
    .attr('x', 30) // Space after the square
    .attr('y', 15)
    .text(
      (d) =>
        `${d.data.Title} - $${xAxisTickFormat(d.data.IMDb_Rating)}`,
    )
    .style('font-size', '15px')
    .style('text-anchor', 'start');
};

// Main function to run the data visualization
const run = async () => {
  const data = await fetchData();
  render(data);
};

run();
